@@ -31,7 +31,11 @@
 #include "bosonplayfield.h"
 #include "global.h"
 #include "upgradeproperties.h"
+#include "bosonprofiling.h"
+#include "bofile.h"
 #include "bodebug.h"
+#include "bosonpropertyxml.h"
+#include "startupwidgets/bosonloadingwidget.h"
 
 #include <klocale.h>
 #include <kgame/kgameio.h>
@@ -43,6 +47,13 @@
 #include <qptrqueue.h>
 #include <qdom.h>
 #include <qdatastream.h>
+#include <qfile.h>
+
+#ifndef KGAME_HAVE_KGAME_PORT
+#include <qsocket.h>
+#include <qserversocket.h>
+#include <qobjectlist.h>
+#endif
 
 #include "boson.moc"
 
@@ -51,7 +62,9 @@ Boson* Boson::mBoson = 0;
 #define ADVANCE_INTERVAL 250 // ms
 
 // Saving format version (000005 = 00.00.05)
-#define BOSON_SAVEGAME_FORMAT_VERSION 000007
+#define BOSON_SAVEGAME_FORMAT_VERSION 0x000041
+
+#define BOSON_SAVEGAME_END_COOKIE 1718
 
 class BoMessage
 {
@@ -174,6 +187,7 @@ public:
 	QPtrList<KGameComputerIO> mComputerIOList;
 	
 	KGamePropertyInt mGameSpeed;
+	KGamePropertyBool mGamePaused;
 
 	KGameProperty<unsigned long int> mNextUnitId;
 	KGameProperty<unsigned int> mAdvanceCount;
@@ -198,10 +212,6 @@ Boson::Boson(QObject* parent) : KGame(BOSON_COOKIE, parent)
 
  connect(this, SIGNAL(signalNetworkData(int, const QByteArray&, Q_UINT32, Q_UINT32)),
 		this, SLOT(slotNetworkData(int, const QByteArray&, Q_UINT32, Q_UINT32)));
- connect(this, SIGNAL(signalSave(QDataStream&)),
-		this, SLOT(slotSave(QDataStream&)));
- connect(this, SIGNAL(signalLoad(QDataStream&)),
-		this, SLOT(slotLoad(QDataStream&)));
  connect(this, SIGNAL(signalReplacePlayerIO(KPlayer*, bool*)),
 		this, SLOT(slotReplacePlayerIO(KPlayer*, bool*)));
  connect(this, SIGNAL(signalPlayerJoinedGame(KPlayer*)),
@@ -214,6 +224,8 @@ Boson::Boson(QObject* parent) : KGame(BOSON_COOKIE, parent)
 		this, SLOT(slotPropertyChanged(KGamePropertyBase*)));
  d->mGameSpeed.registerData(IdGameSpeed, dataHandler(),
 		KGamePropertyBase::PolicyClean, "GameSpeed");
+ d->mGamePaused.registerData(IdGamePaused, dataHandler(),
+		KGamePropertyBase::PolicyClean, "GamePaused");
  d->mNextUnitId.registerData(IdNextUnitId, dataHandler(),
 		KGamePropertyBase::PolicyLocal, "NextUnitId");
  d->mAdvanceCount.registerData(IdAdvanceCount, dataHandler(),
@@ -223,6 +235,7 @@ Boson::Boson(QObject* parent) : KGame(BOSON_COOKIE, parent)
  d->mNextUnitId.setLocal(0);
  d->mAdvanceCount.setLocal(0);
  d->mGameSpeed.setLocal(0);
+ d->mGamePaused.setLocal(false);
  d->mAdvanceFlag.setLocal(0);
  d->mAdvanceCount.setEmittingSignal(false); // wo don't need it and it would be bad for performance.
 
@@ -260,7 +273,12 @@ void Boson::setPlayField(BosonPlayField* p)
  d->mPlayField = p;
 }
 
-Player* Boson::localPlayer()
+BosonPlayField* Boson::playField() const
+{
+ return d->mPlayField;
+}
+
+Player* Boson::localPlayer() const
 {
  return d->mPlayer;
 }
@@ -314,8 +332,17 @@ bool Boson::playerInput(QDataStream& stream, KPlayer* p)
  switch (msgid) {
 	case BosonMessage::MoveMove:
 	{
+		bool attack;
+		Q_UINT8 attackcode;
 		QPoint pos;
 		Q_UINT32 unitCount;
+		stream >> attackcode;
+		if (attackcode == 0) {
+			attack = false;
+		} else {
+			attack = true;
+		}
+		boDebug() << "MOVING: " << k_funcinfo << "attack: " << attack << endl;
 		stream >> pos;
 		stream >> unitCount;
 		QPtrList<Unit> unitsToMove;
@@ -351,11 +378,11 @@ bool Boson::playerInput(QDataStream& stream, KPlayer* p)
 			++it;
 		}
 		if (unitsToMove.count() == 1) {
-			unitsToMove.first()->moveTo(pos);
+			unitsToMove.first()->moveTo(pos, attack);
 		} else {
 			it.toFirst();
 			while (it.current()) {
-				it.current()->moveTo(pos);
+				it.current()->moveTo(pos, attack);
 				++it;
 			}
 		}
@@ -623,8 +650,8 @@ bool Boson::playerInput(QDataStream& stream, KPlayer* p)
 			mineralCost = prop->mineralCost();
 			oilCost = prop->oilCost();
 		} else if ((ProductionType)productionType == ProduceTech) {
-			// Produce technology
-			const TechnologyProperties* prop = p->speciesTheme()->technology(id);
+			// Produce upgrade
+			const UpgradeProperties* prop = p->speciesTheme()->technology(id);
 			if (!prop) {
 				boError() << k_lineinfo << "NULL technology properties (EVIL BUG)" << endl;
 				break;
@@ -647,7 +674,6 @@ bool Boson::playerInput(QDataStream& stream, KPlayer* p)
 		if (p->minerals() < mineralCost) {
 			if (p == localPlayer()) {
 				slotAddChatSystemMessage(i18n("You have not enough minerals!"));
-
 			}
 			break;
 		}
@@ -715,7 +741,7 @@ bool Boson::playerInput(QDataStream& stream, KPlayer* p)
 			mineralCost = prop->mineralCost();
 			oilCost = prop->oilCost();
 		} else if ((ProductionType)productionType == ProduceTech) {
-			const TechnologyProperties* prop = p->speciesTheme()->technology(id);
+			const UpgradeProperties* prop = p->speciesTheme()->technology(id);
 			if (!prop) {
 				boError() << k_lineinfo << "NULL technology properties (EVIL BUG)" << endl;
 				break;
@@ -893,6 +919,33 @@ bool Boson::playerInput(QDataStream& stream, KPlayer* p)
 		}
 		break;
 	}
+	case BosonMessage::MoveChangeHeight:
+	{
+		boDebug() << k_lineinfo << "change height" << endl;
+		Q_UINT32 count;
+		Q_INT32 cornerX;
+		Q_INT32 cornerY;
+		float height;
+		stream >> count;
+		for (unsigned int i = 0; i < count; i++) {
+			stream >> cornerX;
+			stream >> cornerY;
+			stream >> height;
+			// note: cornerX == mapWidth() and cornerY == mapHeight()
+			// are valid!
+			if (cornerX < 0 || (unsigned int)(cornerX - 1) >= d->mCanvas->mapWidth()) {
+				boError() << k_funcinfo << "invalid x coordinate " << cornerX << endl;
+				continue;
+			}
+			if (cornerY < 0 || (unsigned int)(cornerY - 1) >= d->mCanvas->mapHeight()) {
+				boError() << k_funcinfo << "invalid y coordinate " << cornerY << endl;
+				continue;
+			}
+			boDebug() << k_funcinfo << "new height at " << cornerX << "," << cornerY << " is " << height << endl;
+			d->mCanvas->setHeightAtCorner(cornerX, cornerY, height);
+		}
+		break;
+	}
 	default:
 		boWarning() << k_funcinfo << "unexpected playerInput " << msgid << endl;
 		break;
@@ -933,6 +986,7 @@ void Boson::slotNetworkData(int msgid, const QByteArray& buffer, Q_UINT32 , Q_UI
 	{
 		Q_UINT32 owner;
 		stream >> owner;
+		boDebug() << k_lineinfo << "AddUnitsXML for " << owner << endl;
 
 		QString xmlDocument;
 		stream >> xmlDocument;
@@ -948,6 +1002,7 @@ void Boson::slotNetworkData(int msgid, const QByteArray& buffer, Q_UINT32 , Q_UI
 			break;
 		}
 
+		boProfiling->start(BosonProfiling::AddUnitsXML);
 		QDomDocument doc;
 		doc.setContent(xmlDocument);
 		QDomElement root = doc.documentElement();
@@ -956,6 +1011,8 @@ void Boson::slotNetworkData(int msgid, const QByteArray& buffer, Q_UINT32 , Q_UI
 			QDomElement e = list.item(i).toElement();
 			addUnit(e, (Player*)p);
 		}
+		boProfiling->stop(BosonProfiling::AddUnitsXML);
+		boDebug() << k_lineinfo << "AddUnitsXML done" << endl;
 		break;
 	}
 	case BosonMessage::AdvanceN:
@@ -971,6 +1028,11 @@ void Boson::slotNetworkData(int msgid, const QByteArray& buffer, Q_UINT32 , Q_UI
 		slotReceiveAdvance();
 		break;
 	}
+	case BosonMessage::ChangeMap:
+	{
+		emit signalEditorNewMap(buffer);
+		break;
+	}
 	case BosonMessage::InitMap:
 		emit signalInitMap(buffer);
 		break;
@@ -994,9 +1056,16 @@ void Boson::slotNetworkData(int msgid, const QByteArray& buffer, Q_UINT32 , Q_UI
 		setGameMode(false);
 		QTimer::singleShot(0, this, SIGNAL(signalStartNewGame()));
 		break;
+	case BosonMessage::IdStartGameClicked:
+		// this is kind of a workaround.
+		// for --start we need to call slotStart() in the start widgets
+		// only once the (e.g.) playfield messages have arrived. for
+		// this we use a message and *then* call slotStart() there.
+		QTimer::singleShot(0, this, SIGNAL(signalStartGameClicked()));
+		break;
 	case BosonMessage::IdGameIsStarted:
 		if (!isRunning()) {
-			boError() << "Received IdGameIstarted but it isn't" << endl;
+			boError() << "Received IdGameIsStarted but it isn't" << endl;
 			return;
 		}
 		emit signalGameStarted();
@@ -1051,6 +1120,47 @@ void Boson::slotNetworkData(int msgid, const QByteArray& buffer, Q_UINT32 , Q_UI
 	{
 		break;
 	}
+	case BosonMessage::IdKillPlayer:
+	{
+		Player* p = 0;
+		Q_UINT32 id = 0;
+		stream >> id;
+		p = (Player*)findPlayer(id);
+		BO_CHECK_NULL_RET(p);
+		BO_CHECK_NULL_RET(d->mCanvas);
+		d->mCanvas->killPlayer(p);
+		slotAddChatSystemMessage(i18n("Debug"), i18n("Killed player %1 - %2").arg(p->id()).arg(p->name()));
+	}
+	case BosonMessage::IdModifyMinerals:
+	{
+		Player* p = 0;
+		Q_INT32 change = 0;
+		Q_UINT32 id = 0;
+		stream >> id;
+		stream >> change;
+		p = (Player*)findPlayer(id);
+		BO_CHECK_NULL_RET(p);
+		if ((Q_INT32)p->minerals() + change < 0) {
+		}
+		p->setMinerals(p->minerals() + change);
+		break;
+	}
+	case BosonMessage::IdModifyOil:
+	{
+		Player* p = 0;
+		Q_INT32 change = 0;
+		Q_UINT32 id = 0;
+		stream >> id;
+		stream >> change;
+		p = (Player*)findPlayer(id);
+		BO_CHECK_NULL_RET(p);
+		if ((Q_INT32)p->oil() + change < 0) {
+			p->setOil(0);
+		} else {
+			p->setOil(p->oil() + change);
+		}
+		break;
+	}
 	default:
 		boWarning() << k_funcinfo << "unhandled msgid " << msgid << endl;
 		break;
@@ -1105,8 +1215,10 @@ Unit* Boson::createUnit(unsigned long int unitType, Player* owner)
  }
  owner->addUnit(unit); // can also be in Unit c'tor - is this clean?
  theme->loadNewUnit(unit);
+ unit->setAnimationMode(UnitAnimationIdle);
  if (unit->isFlying()) {
-	unit->moveBy(0.0f, 0.0f, 2.0 * BO_TILE_SIZE / BO_GL_CELL_SIZE);
+//	unit->moveBy(0.0f, 0.0f, 2.0 * BO_TILE_SIZE / BO_GL_CELL_SIZE);
+	unit->moveBy(0.0f, 0.0f, 2.0);
  }
 
  return unit;
@@ -1165,10 +1277,9 @@ Unit* Boson::findUnit(unsigned long int id, Player* searchIn) const
  return 0;
 }
 
-KPlayer* Boson::createPlayer(int rtti, int io, bool isVirtual)
+KPlayer* Boson::createPlayer(int , int , bool ) // AB: we don't use these params.
 {
- boDebug() << k_funcinfo << "rtti=" << rtti << ",io=" << io
-		<< ",isVirtual=" << isVirtual << endl;
+ boDebug() << k_funcinfo << endl;
  Player* p = new Player();
  p->setGame(this);
  if (d->mPlayField && d->mPlayField->map()) {
@@ -1185,12 +1296,14 @@ int Boson::gameSpeed() const
  return d->mGameSpeed;
 }
 
+bool Boson::gamePaused() const
+{
+ return d->mGamePaused;
+}
+
 void Boson::slotSetGameSpeed(int speed)
 {
  boDebug() << k_funcinfo << " speed = " << speed << endl;
- if (d->mGameSpeed == speed) {
-	return; // do not restart timer
- }
  if (speed < 0) {
 	boError() << "Invalid speed value " << speed << endl;
 	return;
@@ -1198,12 +1311,21 @@ void Boson::slotSetGameSpeed(int speed)
  if ((speed < MIN_GAME_SPEED || speed > MAX_GAME_SPEED) && speed != 0) {
 	boWarning() << "unexpected speed " << speed << " - pausing" << endl;
 	d->mGameSpeed = 0;
+	// we don't have a manual pause, so don't set d->mGamePaused!
 	return;
  }
  boDebug() << k_funcinfo << "Setting speed to " << speed << endl;
  d->mGameSpeed = speed;
 }
 
+void Boson::slotTogglePause()
+{
+ boDebug() << k_funcinfo << endl;
+ // note that this won't take immediate effect. the variable will change once it
+ // is received from network!
+ d->mGamePaused = !gamePaused();
+}
+
 void Boson::slotPropertyChanged(KGamePropertyBase* p)
 {
  switch (p->id()) {
@@ -1216,7 +1338,7 @@ void Boson::slotPropertyChanged(KGamePropertyBase* p)
 					d->mGameTimer->stop();
 				}
 			} else {
-				if (!d->mGameTimer->isActive()) {
+				if (!d->mGameTimer->isActive() && !gamePaused()) {
 					boDebug() << "start timer - ms="
 							<< ADVANCE_INTERVAL
 							<< endl;
@@ -1224,22 +1346,33 @@ void Boson::slotPropertyChanged(KGamePropertyBase* p)
 				}
 			}
 		}
+		if (gamePaused()) {
+			boProfiling->setGameSpeed(0);
+		} else {
+			boProfiling->setGameSpeed(gameSpeed());
+		}
+		break;
+	case IdGamePaused:
+		boDebug() << k_funcinfo << "game paused changed! now=" << d->mGamePaused << endl;
+		if (d->mGamePaused) {
+			slotAddChatSystemMessage(i18n("The game is paused now!"));
+			d->mGameTimer->stop();
+		} else if (d->mGameSpeed > 0) {
+			boDebug() << k_funcinfo << "starting timer again" << endl;
+			slotAddChatSystemMessage(i18n("The game is not paused anymore"));
+			d->mGameTimer->start(ADVANCE_INTERVAL);
+		}
+		if (gamePaused()) {
+			boProfiling->setGameSpeed(0);
+		} else {
+			boProfiling->setGameSpeed(gameSpeed());
+		}
 		break;
 	default:
 		break;
  }
 }
 
-void Boson::slotSave(QDataStream& /*stream*/)
-{ // save non-KGameProperty datas here
-// stream <<
-}
-
-void Boson::slotLoad(QDataStream& /*stream*/)
-{
-// boDebug() << "next id: " << d->mNextUnitId << endl;
-}
-
 void Boson::slotSendAddUnit(unsigned long int unitType, int x, int y, Player* owner)
 { // used by the editor directly
  if (!isServer()) {
@@ -1308,7 +1441,7 @@ bool Boson::buildProducedUnit(ProductionPlugin* factory, unsigned long int unitT
 	boError() << k_funcinfo << "NULL owner" << endl;
 	return false;
  }
- if (!((BosonCanvas*)d->mCanvas)->canPlaceUnitAt(p->unitProperties(unitType), QPoint(x, y), 0)) {
+ if (!(d->mCanvas)->canPlaceUnitAtCell(p->unitProperties(unitType), QPoint(x, y), 0)) {
 	boDebug() << k_funcinfo << "Cannot create unit here" << endl;
 	return false;
  }
@@ -1350,6 +1483,15 @@ Unit* Boson::addUnit(unsigned long int unitType, Player* p, int x, int y)
  }
  unit->setId(nextUnitId());
  emit signalAddUnit(unit, x * BO_TILE_SIZE, y * BO_TILE_SIZE);
+ if (!gameMode()) {
+	// editor won't display the construction, but always completed
+	// facilities. otherwise it's hard to recognize where they were actually
+	// placed
+	if (unit->glConstructionSteps() > 0) {
+		unit->setGLConstructionStep(unit->glConstructionSteps());
+	}
+	unit->setAnimationMode(UnitAnimationIdle);
+ }
  return unit;
 }
 
@@ -1374,13 +1516,23 @@ Unit* Boson::addUnit(QDomElement& node, Player* p)
 	// the player will never know that we had a problem here. Just a few
 	// (non-critical) values were not loaded.
  }
- 
+
  emit signalAddUnit(unit, x * BO_TILE_SIZE, y * BO_TILE_SIZE);
+ if (!gameMode()) {
+	// editor won't display the construction, but always completed
+	// facilities. otherwise it's hard to recognize where they were actually
+	// placed
+	if (unit->glConstructionSteps() > 0) {
+		unit->setGLConstructionStep(unit->glConstructionSteps());
+	}
+	unit->setAnimationMode(UnitAnimationIdle);
+ }
  return unit;
 }
 
 void Boson::slotPlayerJoinedGame(KPlayer* p)
 {
+ boDebug() << k_funcinfo << endl;
  if (!p) {
 	return;
  }
@@ -1390,6 +1542,11 @@ void Boson::slotPlayerJoinedGame(KPlayer* p)
 	d->mComputerIOList.append((KGameComputerIO*)io);
  }
  slotAddChatSystemMessage(i18n("Player %1 - %2 joined").arg(p->id()).arg(p->name()));
+
+ if (!localPlayer() && playerCount() == 1) {
+	boWarning() << k_funcinfo << "first player entered the game and no localplayer is set - assume that the localplayer entered" << endl;
+	setLocalPlayer((Player*)p);
+ }
 }
 
 void Boson::slotPlayerLeftGame(KPlayer* p)
@@ -1465,7 +1622,7 @@ void Boson::slotReceiveAdvance()
  // - e.g. unit moving, OpenGL rendering, ... so 
  if (d->mAdvanceDividerCount + 1 == d->mAdvanceDivider)  {
 	boDebug(300) << k_funcinfo << "delayed messages: "
-			<< d->mDelayedMessages.count() << endl;
+			<< delayedMessageCount() << endl;
 	unlock();
  } else if (d->mAdvanceDividerCount + 1 < d->mAdvanceDivider) {
 	int next;
@@ -1476,7 +1633,7 @@ void Boson::slotReceiveAdvance()
 	} else {
 		next = 0;
 	}
-	if (d->mDelayedMessages.count() > 20) {
+	if (delayedMessageCount() > 20) {
 		boWarning() << k_funcinfo << "more than 20 messages delayed!!" << endl;
 		next = 0;
 	}
@@ -1549,6 +1706,404 @@ void Boson::slotProcessDelayed() // TODO: rename: processDelayed()
  delete m;
 }
 
+bool Boson::saveToFile(const QString& file)
+{
+ boDebug() << k_funcinfo << file << endl;
+ BosonProfiler profiler(BosonProfiling::SaveGameToXML);
+ if (playerCount() < 1) {
+	boError() << k_funcinfo << "no players in game. cannot save" << endl;
+	return false;
+ }
+ if (gameStatus() == KGame::Init) {
+	boError() << k_funcinfo << "Running game must not be in Init state" << endl;
+	return false;
+ }
+ if (!d->mPlayField) {
+	BO_NULL_ERROR(d->mPlayField);
+	return false;
+ }
+ boProfiling->start(BosonProfiling::SaveKGameToXML);
+ QString kgameXML = saveKGameAsXML();
+ boProfiling->stop(BosonProfiling::SaveKGameToXML);
+ if (kgameXML.isNull()) {
+	return false;
+ }
+
+ boProfiling->start(BosonProfiling::SavePlayersToXML);
+ QString playersXML = savePlayersAsXML();
+ boProfiling->stop(BosonProfiling::SavePlayersToXML);
+ if (playersXML.isNull()) {
+	return false;
+ }
+
+ boProfiling->start(BosonProfiling::SaveCanvasToXML);
+ QString canvasXML = saveCanvasAsXML();
+ boProfiling->stop(BosonProfiling::SaveCanvasToXML);
+ if (canvasXML.isNull()) {
+	return false;
+ }
+
+ boProfiling->start(BosonProfiling::SaveExternalToXML);
+ QString externalXML = saveExternalAsXML();
+ boProfiling->stop(BosonProfiling::SaveExternalToXML);
+ if (externalXML.isNull()) {
+	return false;
+ }
+
+
+
+ // we store the map as binary. XML would take a lot of space and time to save
+ // and load.
+ QByteArray map;
+ QDataStream stream(map, IO_WriteOnly);
+ boProfiling->start(BosonProfiling::SavePlayFieldToXML);
+ d->mPlayField->saveMap(stream);
+ d->mPlayField->saveDescription(stream);
+ boProfiling->stop(BosonProfiling::SavePlayFieldToXML);
+
+ BosonProfiler writeProfiler(BosonProfiling::SaveGameToXMLWriteFile);
+ BSGFile f(file, false);
+ if (!f.writeFile(QString::fromLatin1("kgame.xml"), kgameXML)) {
+	boError() << k_funcinfo << "Could not write kgame.xml to " << file << endl;
+	return false;
+ }
+ if (!f.writeFile(QString::fromLatin1("players.xml"), playersXML)) {
+	boError() << k_funcinfo << "Could not write players.xml to " << file << endl;
+	return false;
+ }
+ if (!f.writeFile(QString::fromLatin1("canvas.xml"), canvasXML)) {
+	boError() << k_funcinfo << "Could not write canvas.xml to " << file << endl;
+	return false;
+ }
+ if (!f.writeFile(QString::fromLatin1("external.xml"), externalXML)) {
+	boError() << k_funcinfo << "Could not write external.xml to " << file << endl;
+	return false;
+ }
+ if (!f.writeFile(QString::fromLatin1("map"), map)) {
+	boError() << k_funcinfo << "Could not write map to " << file << endl;
+	return false;
+ }
+ writeProfiler.stop(); // in case we add something below one day :)
+
+ return true;
+}
+
+QString Boson::saveKGameAsXML()
+{
+ QDomDocument doc(QString::fromLatin1("Boson"));
+ QDomElement root = doc.createElement(QString::fromLatin1("Boson")); // XML file for the Boson object
+ doc.appendChild(root);
+ root.setAttribute(QString::fromLatin1("SaveGameVersion"), BOSON_SAVEGAME_FORMAT_VERSION);
+
+ // store the dataHandler()
+ BosonCustomPropertyXML propertyXML;
+ QDomElement handler = doc.createElement(QString::fromLatin1("DataHandler"));
+ if (!propertyXML.saveAsXML(handler, dataHandler())) { // AB: we should exclude gameStatus from this! we should stay in KGame::Init! -> add a BosonPropertyXML::remove() or so
+	boError() << k_funcinfo << "unable to save KGame data handler" << endl;
+	return QString::null;
+ }
+ root.appendChild(handler);
+
+ // here we could add additional data for the Boson object.
+ // but I believe all data should get into the datahandler as far as possible
+
+ return doc.toString();
+}
+
+QString Boson::savePlayersAsXML()
+{
+ QDomDocument doc(QString::fromLatin1("Players"));
+ QDomElement root = doc.createElement(QString::fromLatin1("Players"));
+ doc.appendChild(root);
+
+ KGamePlayerList* list = playerList();
+ for (KPlayer* p = list->first(); p; p = list->next()) {
+	// KGame also stored ID, RTTI and KPlayer::calcIOValue() here.
+	// I believe we won't need them. ID might be useful for network games,
+	// but we load from a file here.
+	QDomElement element = doc.createElement(QString::fromLatin1("Player"));
+	if (!((Player*)p)->saveAsXML(element)) {
+		boError() << k_funcinfo << "Unable to save player " << p->id() << endl;
+		continue;
+	}
+	root.appendChild(element);
+ }
+
+ // save the ID of the local player
+ if (d->mPlayer) {
+	root.setAttribute(QString::fromLatin1("LocalPlayerId"), (unsigned int)d->mPlayer->id());
+ } else {
+	// this might be the case for network games.
+	// (when a player enters the game it is saved on the ADMIN and loaded on
+	// the new client).
+	// But currently we don't use XML for that. Maybe we will never do.
+	boWarning() << k_funcinfo << "NULL local player" << endl;
+ }
+
+ return doc.toString();
+}
+
+QString Boson::saveCanvasAsXML()
+{
+ QDomDocument doc(QString::fromLatin1("Canvas"));
+ QDomElement root = doc.createElement(QString::fromLatin1("Canvas")); // XML file for canvas
+ doc.appendChild(root);
+
+ d->mCanvas->saveAsXML(root);
+
+ return doc.toString();
+}
+
+QString Boson::saveExternalAsXML()
+{
+ QDomDocument doc(QString::fromLatin1("External"));
+ QDomElement root = doc.createElement(QString::fromLatin1("External")); // XML file for external data
+ doc.appendChild(root);
+
+ emit signalSaveExternalStuffAsXML(root);
+
+ return doc.toString();
+}
+
+bool Boson::loadFromFile(const QString& file)
+{
+ boDebug(260) << k_funcinfo << endl;
+ d->mLoadingStatus = LoadingInProgress;
+ BSGFile f(file, true);
+ if (!f.checkTar()) {
+	boError(260) << k_funcinfo << "Could not load from " << file << endl;
+	d->mLoadingStatus = BSGFileError;
+	return false;
+ }
+ QString kgameXML;
+ QString playersXML;
+ QString canvasXML;
+ QString externalXML;
+ QByteArray map;
+
+ kgameXML = QString(f.kgameData());
+ playersXML = QString(f.playersData());
+ canvasXML = QString(f.canvasData());
+ externalXML = QString(f.externalData());
+ map = f.mapData();
+
+ if (kgameXML.isEmpty()) {
+	boError(260) << k_funcinfo << "Empty kgameXML" << endl;
+	d->mLoadingStatus = BSGFileError;
+	return false;
+ }
+ if (playersXML.isEmpty()) {
+	boError(260) << k_funcinfo << "Empty playersXML" << endl;
+	d->mLoadingStatus = BSGFileError;
+	return false;
+ }
+ if (canvasXML.isEmpty()) {
+	boError(260) << k_funcinfo << "Empty canvasXML" << endl;
+	d->mLoadingStatus = BSGFileError;
+	return false;
+ }
+ if (externalXML.isEmpty()) {
+	boError(260) << k_funcinfo << "Empty externalXML" << endl;
+	d->mLoadingStatus = BSGFileError;
+	return false;
+ }
+ if (map.isEmpty()) {
+	boError(260) << k_funcinfo << "Empty map" << endl;
+	d->mLoadingStatus = BSGFileError;
+	return false;
+ }
+
+ reset();
+ if (playerCount() != 0) {
+	boError(260) << k_funcinfo << "not all players removed!" << endl;
+	d->mLoadingStatus = BSGFileError;
+	return false;
+ }
+
+ if (!loadKGameFromXML(kgameXML)) {
+	return false;
+ }
+
+ // KGame::loadgame() also loads KGame::d->mUniquePlayerNumber !!
+ // KGame::loadgame() also loads a seed for KGame::random() (not so important)
+
+ // now load the players (not the units!)
+ QMap<Player*, QDomElement> player2Element;
+ QDomDocument doc(QString::fromLatin1("Boson"));
+ if (!loadXMLDoc(&doc, playersXML)) {
+	d->mLoadingStatus = InvalidXML;
+	return false;
+ }
+ QDomElement root = doc.documentElement();
+ if (!root.hasAttribute(QString::fromLatin1("LocalPlayerId"))) {
+	boError(260) << k_funcinfo << "missing attribute: LocalPlayerId" << endl;
+	d->mLoadingStatus = InvalidXML;
+	return false;
+ }
+ bool ok = false;
+ unsigned int localId = root.attribute(QString::fromLatin1("LocalPlayerId")).toUInt(&ok);
+ if (!ok) {
+	boError(260) << k_funcinfo << "invalid LocalPlayerId: " << root.attribute(QString::fromLatin1("LocalPlayerId")) << endl;
+	d->mLoadingStatus = InvalidXML;
+	return false;
+ }
+ QDomNodeList list = root.elementsByTagName(QString::fromLatin1("Player"));
+ if (list.count() < 1) {
+	boError(260) << k_funcinfo << "no Player tags in file" << endl;
+	d->mLoadingStatus = InvalidXML;
+	return false;
+ }
+ boDebug(260) << k_funcinfo << "loading " << list.count() << " players" << endl;
+ emit signalLoadingPlayersCount(list.count());
+ for (unsigned int i = 0; i < list.count(); i++) {
+	boDebug(260) << k_funcinfo << "creating player " << i << endl;
+	QDomElement player = list.item(i).toElement();
+	if (player.isNull()) {
+		boError(260) << k_funcinfo << "NULL player node" << endl;
+		continue;
+	}
+	Player* p = (Player*)createPlayer(0, 0, false); // we ignore all params anyway.
+	p->loadFromXML(player);
+	systemAddPlayer((KPlayer*)p);
+	player2Element.insert(p, player);
+ }
+ boDebug(260) << k_funcinfo << "searching local player" << endl;
+ d->mPlayer = (Player*)findPlayer(localId);
+ if (!d->mPlayer) {
+	boWarning(260) << k_funcinfo << "local player NOT found" << endl;
+ }
+
+
+ // now load the map (must happen before loading units)
+ // AB: can we move this before loading the players?
+ emit signalLoadingType(BosonLoadingWidget::ReceiveMap);
+ emit signalInitMap(map);
+
+
+ // Load player data (e.g. unit configs, unit models, ...
+ // TODO: since we use XML for savegames now we should be able to include this
+ // to regular startup code in BosonStarting!
+ KPlayer* p;
+ int current = 0;
+ for (p = playerList()->first(); p; p = playerList()->next(), current++) {
+	emit signalLoadingPlayer(current);
+	emit signalLoadPlayerData((Player*)p);
+	emit signalLoadingType(BosonLoadingWidget::LoadSavedUnits);
+ }
+
+
+ boDebug(260) << k_funcinfo << "loading units" << endl;
+ QMap<Player*, QDomElement>::Iterator it;
+ for (it = player2Element.begin(); it != player2Element.end(); ++it) {
+	Player* p = it.key();
+	if (!p->speciesTheme()) {
+		boError() << k_funcinfo << "NULL speciesTheme" << endl;
+		continue;
+	}
+	QDomElement e = it.data();
+	p->loadUnitsFromXML(e);
+ }
+
+ // Load canvas (shots)
+ if (!loadCanvasFromXML(canvasXML)) {
+	return false;
+ }
+
+ // Load external stuff (camera)
+ if (!loadExternalFromXML(externalXML)) {
+	return false;
+ }
+
+ d->mLoadingStatus = LoadingCompleted;
+ return true;
+}
+
+bool Boson::loadKGameFromXML(const QString& xml)
+{
+ boDebug() << k_funcinfo << endl;
+ QDomDocument doc(QString::fromLatin1("Boson"));
+ if (!loadXMLDoc(&doc, xml)) {
+	d->mLoadingStatus = InvalidXML;
+	return false;
+ }
+ QDomElement root = doc.documentElement();
+ bool ok = false;
+ unsigned int version = root.attribute(QString::fromLatin1("SaveGameVersion")).toUInt(&ok);
+ if (!ok) {
+	boError() << k_funcinfo << "savegame version not a valid number: " << root.attribute(QString::fromLatin1("SaveGameVersion")) << endl;
+	d->mLoadingStatus = InvalidXML;
+	return false;
+ }
+ if (version != latestSavegameVersion()) {
+	boError() << "savegame version " << version << " not supported (latest is " << latestSavegameVersion() << ")" << endl;
+	d->mLoadingStatus = InvalidVersion;
+	return false;
+ }
+
+ // load the datahandler
+ QDomElement handler = root.namedItem(QString::fromLatin1("DataHandler")).toElement();
+ if (handler.isNull()) {
+	boError() << k_funcinfo << "No DataHandler tag found" << endl;
+	d->mLoadingStatus = InvalidXML;
+	return false;
+ }
+ BosonCustomPropertyXML propertyXML;
+ if (!propertyXML.loadFromXML(handler, dataHandler())) {
+	boError() << k_funcinfo << "unable to load KGame data handler" << endl;
+	d->mLoadingStatus = InvalidXML;
+	return false;
+ }
+#warning dont load gamestatus!
+ // FIXME: the property handler also loads KGame::gameStatus(), but it must
+ // remain in Init state until we completed loading!
+ // this is a workaround for this problem:
+ {
+	// set gameStatus to Init. Will be set to Run later
+	QByteArray b;
+	QDataStream s(b, IO_WriteOnly);
+	KGameMessage::createPropertyHeader(s, KGamePropertyBase::IdGameStatus);
+	s << (int)KGame::Init;
+	QDataStream readStream(b, IO_ReadOnly);
+	dataHandler()->processMessage(readStream, dataHandler()->id(), false);
+ }
+
+ return true;
+}
+
+bool Boson::loadCanvasFromXML(const QString& xml)
+{
+ boDebug() << k_funcinfo << endl;
+ QDomDocument doc(QString::fromLatin1("Canvas"));
+ if (!loadXMLDoc(&doc, xml)) {
+	d->mLoadingStatus = InvalidXML;
+	return false;
+ }
+ QDomElement root = doc.documentElement();
+
+ if (!d->mCanvas->loadFromXML(root)) {
+	d->mLoadingStatus = InvalidXML;
+	return false;
+ }
+
+ return true;
+}
+
+bool Boson::loadExternalFromXML(const QString& xml)
+{
+ boDebug() << k_funcinfo << endl;
+ // Load external stuff (camera)
+ QDomDocument doc(QString::fromLatin1("External"));
+ if (!loadXMLDoc(&doc, xml)) {
+	d->mLoadingStatus = InvalidXML;
+	return false;
+ }
+ QDomElement root = doc.documentElement();
+
+ emit signalLoadExternalStuffFromXML(root);
+
+ return true;
+}
+
 bool Boson::save(QDataStream& stream, bool saveplayers)
 {
 #if HAVE_KGAME_SAVEGAME
@@ -1563,6 +2118,19 @@ bool Boson::save(QDataStream& stream, bool saveplayers)
 
 bool Boson::savegame(QDataStream& stream, bool network, bool saveplayers)
 {
+ // AB: we need to save:
+ // - cookie or something similar (to identify a boson non-kgame savegame)
+ // - version of the savegame format
+ // - KGame stores KGame::d->mUniquePlayerNumber. do we need to, too? i believe
+ // not!
+ // - KGame::d->mRandom->seed() !
+ // - KGame::dataHandler() (use dataHandler()->save() or so. try to use xml)
+ // - playerCount
+ // - players
+ //
+ // + boson relevant data
+
+
  boDebug() << k_funcinfo << endl;
  // KGame::load() doesn't emit signalLoadPrePlayers in KDE 3.0.x, so we have to
  //  rewrite some code to be able to load map before players (because players
@@ -1573,15 +2141,19 @@ bool Boson::savegame(QDataStream& stream, bool network, bool saveplayers)
  stream << (Q_UINT8)128;
  stream << (Q_UINT8)'B' << (Q_UINT8)'S' << (Q_UINT8)'G';  // BSG = Boson SaveGame
  // Magic cookie
- stream << (Q_UINT32)cookie();
+ stream << (Q_INT32)cookie();
  // Version information (for future format changes and backwards compatibility)
  stream << (Q_UINT32)BOSON_SAVEGAME_FORMAT_VERSION;
 
+ // Players count for loading progressbar
+ stream << (Q_UINT32)playerList()->count();
+
  if (gameStatus() != KGame::Init) {
 	boDebug() << k_funcinfo << "Saveing started game" << endl;
 	stream << (Q_INT8)true;
 	// Save map
 	d->mPlayField->saveMap(stream);
+	d->mPlayField->saveDescription(stream);
 
 	// Save local player (only id)
 	stream << d->mPlayer->id();
@@ -1610,6 +2182,18 @@ bool Boson::savegame(QDataStream& stream, bool network, bool saveplayers)
 	}
  }
 
+ if (gameStatus() != KGame::Init) {
+	// Save external stuff (camera, shots, particle systems...)
+	if (!d->mCanvas->save(stream)) {
+		boError() << k_funcinfo << "Error when saving canvas" << endl;
+	}
+	emit signalSaveExternalStuff(stream);
+ }
+
+ // Save end cookie
+ stream << (Q_UINT32)BOSON_SAVEGAME_END_COOKIE;
+
+
  boDebug() << k_funcinfo << " done" << endl;
  return true;
 }
@@ -1620,39 +2204,48 @@ bool Boson::load(QDataStream& stream, bool reset)
  return loadgame(stream, false, reset);
 }
 
+// AB: call KGame::reset() on loading!
 bool Boson::loadgame(QDataStream& stream, bool network, bool reset)
 {
  // network is false for normal game-loading
  boDebug() << k_funcinfo << endl;
 
  d->mLoadingStatus = LoadingInProgress;
+ emit signalLoadingType(BosonLoadingWidget::LoadSavedGameHeader);
 
  // Load magic data
  Q_UINT8 a, b1, b2, b3;
- Q_INT32 c, v;
+ Q_INT32 c;
+ Q_UINT32 v;
  stream >> a >> b1 >> b2 >> b3;
  if ((a != 128) || (b1 != 'B' || b2 != 'S' || b3 != 'G')) {
 	// Error - not Boson SaveGame
-	boError() << k_funcinfo << "This file is not Boson SaveGame" << endl;
+	boError() << k_funcinfo << "This file is no Boson SaveGame" << endl;
 	d->mLoadingStatus = InvalidFileFormat;
 	return false;
  }
  stream >> c;
  if (c != cookie()) {
 	// Error - wrong cookie
-	boError() << k_funcinfo << "Invalid cookie in header" << endl;
+	boError() << k_funcinfo << "Invalid cookie in header (found: " << c << "; should be: " << cookie() << ")" << endl;
 	d->mLoadingStatus = InvalidCookie;
 	return false;
  }
  stream >> v;
- if (v != BOSON_SAVEGAME_FORMAT_VERSION) {
+ if (v != latestSavegameVersion()) {
 	// Error - older version
 	// TODO: It may be possible to load this version
-	boError() << k_funcinfo << "Unsupported format version" << endl;
+	boError() << k_funcinfo << "Unsupported format version (found: " << v << "; latest: " << latestSavegameVersion() << ")" << endl;
 	d->mLoadingStatus = InvalidVersion;
 	return false;
  }
 
+ // Players count for loading progressbar
+ Q_UINT32 playerscount;
+ stream >> playerscount;
+ emit signalLoadingPlayersCount(playerscount);
+
+ emit signalLoadingType(BosonLoadingWidget::LoadSavedGame);
  Q_UINT32 localId = 0;
  Q_INT8 started; // network players usually connect before starting a game
  stream >> started;
@@ -1665,8 +2258,13 @@ bool Boson::loadgame(QDataStream& stream, bool network, bool reset)
 		boError() << k_funcinfo << "Could not load map" << endl;
 		return false;
 	}
+	if (!f->loadDescription(stream)) {
+		boError() << k_funcinfo << "Could not load map description" << endl;
+		return false;
+	}
 	QDataStream mapstream(mapBuffer, IO_WriteOnly);
 	f->saveMap(mapstream);
+	f->saveDescription(mapstream);
 	delete f;
 
 	// Load local player's id
@@ -1681,6 +2279,7 @@ bool Boson::loadgame(QDataStream& stream, bool network, bool reset)
 	d->mLoadingStatus = KGameError;
 	return false;
  }
+ boDebug() << k_funcinfo << "kgame loading successful" << endl;
 
  // KGame::loadgame() also loads the gamestatus. some functions depend on KGame
  // to be in Init status as long as it is still loading, so we set it manually
@@ -1698,30 +2297,41 @@ bool Boson::loadgame(QDataStream& stream, bool network, bool reset)
  }
 
  if (started) {
+	// Set local player. This has to be done before loading units
+	d->mPlayer = (Player*)findPlayer(localId);
 	// AB: needs to be emitted after KGame::loadgame() which adds the
 	// players
+	emit signalLoadingType(BosonLoadingWidget::ReceiveMap);
 	emit signalInitMap(mapBuffer);
  }
 
  // units. they must be loaded *after* the players
  QPtrListIterator<KPlayer> it(*playerList());
- for (; it.current(); ++it) {
-	// Order of calls below is very important!!! Don't change this unless you're sure you know what you're doing!!!
-	((Player*)it.current())->speciesTheme()->loadParticleSystems();
-	((Player*)it.current())->speciesTheme()->readUnitConfigs();
-//	loadUnitDatas(((Player*)it.current()), progress);
-	((Player*)it.current())->speciesTheme()->loadTechnologies();
+ int currentplayer = 0;
+ for (; it.current(); ++it, currentplayer++) {
+	emit signalLoadingPlayer(currentplayer);
+	emit signalLoadPlayerData((Player*)it.current());
+	emit signalLoadingType(BosonLoadingWidget::LoadSavedUnits);
 	if (!((Player*)it.current())->loadUnits(stream)) {
 		boError() << k_funcinfo << "Error when loading units" << endl;
 		return false;
 	}
  }
 
- boDebug() << k_funcinfo << "kgame loading successful" << endl;
+ if (started) {
+	// Load external stuff (camera, shots, particle systems...)
+	if (!d->mCanvas->load(stream)) {
+		boError() << k_funcinfo << "Error when loading canvas" << endl;
+	}
+	emit signalLoadExternalStuff(stream);
+ }
 
- if (started) { // AB (02/09/04): by any reason this was "!started" before - can't work, as localId is not initialized then. is this fix correct?
-	// Set local player
-	d->mPlayer = (Player*)findPlayer(localId);
+ // Check end cookie
+ Q_UINT32 endcookie;
+ stream >> endcookie;
+ if (endcookie != BOSON_SAVEGAME_END_COOKIE) {
+	boError() << k_funcinfo << "Invalid end cookie!" << endl;
+	return false;
  }
 
  d->mLoadingStatus = LoadingCompleted;
@@ -1734,6 +2344,11 @@ Boson::LoadingStatus Boson::loadingStatus() const
  return d->mLoadingStatus;
 }
 
+unsigned long int Boson::latestSavegameVersion()
+{
+ return BOSON_SAVEGAME_FORMAT_VERSION;
+}
+
 void Boson::toggleAdvanceFlag()
 {
  d->mAdvanceFlag = !d->mAdvanceFlag;
@@ -1765,3 +2380,54 @@ void Boson::slotDebugOutput(const QString& area, const char* data, int level)
  slotAddChatSystemMessage(area, data);
 }
 
+unsigned int Boson::delayedMessageCount() const
+{
+ return d->mDelayedMessages.count();
+}
+
+bool Boson::loadXMLDoc(QDomDocument* doc, const QString& xml)
+{
+ QString errorMsg;
+ int lineNo, columnNo;
+ if (!doc->setContent(xml, &errorMsg, &lineNo, &columnNo)) {
+	boError() << k_funcinfo << "Parse error in line " << lineNo << ",column " << columnNo
+			<< " error message: " << errorMsg << endl;
+	return false;
+ }
+ return true;
+}
+
+Q_UINT16 Boson::bosonPort()
+{
+#ifdef KGAME_HAVE_KGAME_PORT
+ return KGame::port();
+#else
+ if (!isNetwork()) {
+	return 0;
+ }
+ boDebug() << k_funcinfo << endl;
+ // ugly hack... luckily we *can* do such hacks :)
+ const QObjectList* list = QObject::objectTrees();
+ QObjectListIt it(*list);
+ if (isOfferingConnections()) {
+	for (; it.current(); ++it) {
+		if (qstrcmp((*it)->className(), "KMessageServerSocket") == 0) {
+			QServerSocket* server = (QServerSocket*)(*it);
+			return server->port();
+		}
+	}
+	boWarning() << k_funcinfo << "could not find KMessageServerSocket!" << endl;
+ } else {
+	for (; it.current(); ++it) {
+		if (qstrcmp((*it)->className(), "QSocket") == 0) {
+			QSocket* socket = (QSocket*)*it;
+			return socket->peerPort();
+
+		}
+	}
+	boWarning() << k_funcinfo << "could not find QSocket!" << endl;
+ }
+ return 0;
+#endif
+}
+
